%% fit transfer function (alpha)

clearvars
close all

% Getting all the directories of recorded cells that contain the transfer
% function protocol and deleting unnecessary folders
directories = dir('/Users/stefanobordabossana/Documents/Experiments/analysed cells with tf/*');
directories(1:3) = [];
directories(end) = [];

for i = 1:length(directories) % For loop over all the recorded cells
    
    cd([directories(i).folder '/' directories(i).name]);
    
    % loading the tf structure with all the necessary data
    load([directories(i).folder '/' directories(i).name '/tf/tf.mat']);
    
    frange = [1:0.3:1000]'; % Defining the appropriate frequency range for the transfer function
    
    % Finding the appropriate indexes to use for the fitting
    fitind2 = find(frange > (tf.cutoff + 2/10*tf.cutoff), 1, 'first');
    fitind1 = find(frange < (tf.cutoff - 0/10*tf.cutoff), 1, 'last');
    
    % Extracting the range to fit
    xfit = frange(fitind1:fitind2);
    yfit = tf.TF(fitind1:fitind2);
    
    ft=fittype('(b/(x^(a)))', 'independent', 'x', 'dependent', 'y'); % Defining power law
    % Defining some fitting parameters
    opts = fitoptions('Method', 'NonlinearLeastSquares');
    opts.Display = 'Off';
    opts.StartPoint = [1, 1];
    [fitresult, gof] = fit(xfit, yfit, ft, opts); % Fitting the data points
    coeff = coeffvalues(fitresult);
    fitX = min(xfit):0.01:max(xfit);
    fitY = coeff(2)./ fitX .^ coeff(1);
    
    % Saving the necessary values in the tf-structure
    tf.slope_fittedtf = coeff(1);
    tf.fittf = fitresult;
    tf.fittf_coeff = coeff;
    
    pathtosave = [pwd '/tf'];
    save([pathtosave '/tf.mat'], 'tf'); % Saving the tf-structure
    
end % Loop over all cells
