clearvars
close all

% Loading all the traces
kernels = dir('*_kernel.dat');
Ke = load(kernels(1).name);

files = listH5Files;

[t,Vr,Iinj,stim] = loadH5Traces(files);

[Vc] = myAEC(Ke, Iinj, Vr, files, t, stim, 1);
Iinj_step = Iinj;

%% 
% check resting potential, input resistance and firing rate stability
% find AP timings

[entitites, info] = loadH5Trace(files{1});

% Initializing variables
spikecounter = 0;
V0 = zeros(length(files), 1);
FiringFrequency_test = zeros(length(files), 1);
FiringFrequency = zeros(length(files), 1);
for l = 1:length(files) % Loop over different recordings
    
    Iinj_step = Iinj(l,:); % Injected current
    Iinj_step(Iinj_step ~= 0) = 1;
    % Start/end of different stimulations
    stimulation_StartPoints = strfind(Iinj_step, [0 1]);
    stimulation_EndPoints = strfind(Iinj_step, [1 0]);

    V0(l,:) = mean(Vc(l,1:stimulation_StartPoints(1)));

    test_min_ind = find(Vc(l,:) == min(Vc(l,:)), 1, 'first');
    voltage_test = Vc(l, test_min_ind);
    Iinj_test = mean(Iinj(l, stimulation_StartPoints(1)+1:stimulation_EndPoints(1)));

    R(l,:) = voltage_test / Iinj_test;

    f_test_duration = t(stimulation_EndPoints(2)) - t(stimulation_StartPoints(2)); % Total time of stimulation
    [AP_test_pks, AP_test_locs] = findpeaks(Vc(l,stimulation_StartPoints(2):stimulation_EndPoints(2)), 'MinPeakHeight', -20, 'MinPeakWidth', 20); % Locate the AP

    FiringFrequency_test(l,:) = length(AP_test_locs) / f_test_duration; % Calculate firing frequency for stability
    
    [~, AP_locs] = findpeaks(Vc(l,stimulation_StartPoints(3)+1:stimulation_EndPoints(3)), 'MinPeakHeight', -20, 'MinPeakWidth', 20);
    FiringFrequency(l,:) = length(AP_locs) / (t(stimulation_EndPoints(3)) - t(stimulation_StartPoints(3)+1)); % Firing frequency during noisy current injection
    
    Inoise(l,:) = Iinj(l ,stimulation_StartPoints(3)+1:stimulation_EndPoints(3)) ...
            - mean(Iinj(l,stimulation_StartPoints(3)+1:stimulation_EndPoints(3))); % Noisy current
    
    % Deleting AP closer than 500 ms to start or end of stimulation, for
    % boundary reasons later in calculation
    SpikeTrain = AP_locs(AP_locs > 0.5*info.srate & AP_locs < length(Inoise) - 0.5*info.srate); 

    spikecounter = spikecounter + size(SpikeTrain, 2); % Keeping track of total number of AP
    
end

%% Calculating iac and sta 

% Initializing variables
randtoshift = randi(200, 500, 1);
stavector = zeros(spikecounter, info.srate + 1);
shiftedstavector = zeros(info.srate+1, size(randtoshift, 1));
counter = 0;
lagsvector = zeros(length(files), info.srate + 1);
cvector = zeros(length(files), info.srate + 1);
for k = 1:length(files) % Loop over different recordings
    
    if R(k) < R(1) + (0.30 * R(1)) && R(k) > R(1) - (0.30 * R(1)) % Check for stability of experiment
        
        % Calculating the input current autocorrelation
        [c, lags] = xcorr(Inoise(k,:), Inoise(k,:));
        lags = lags * info.dt;
        zero = find(lags == 0);
        lagsvector(k,:) = lags(zero - 0.5*info.srate : zero + 0.5*info.srate);
        cvector(k,:) = c(zero - 0.5*info.srate : zero + 0.5*info.srate);

        % Calculating the spike triggered average
        [~, AP_locs] = findpeaks(Vc(k,stimulation_StartPoints(3)+1:stimulation_EndPoints(3)), 'MinPeakHeight', -20, 'MinPeakWidth', 20);
        SpikeTrain = AP_locs(AP_locs > round(0.5*info.srate) & AP_locs < round(length(Inoise) - 0.5*info.srate));
        spikeTimings = SpikeTrain * info.dt;
        isi = diff(spikeTimings);
        
        % Bootstrapping to calculate significance level
        
        for q = 1:size(randtoshift, 1) % Loop over the amount of times needed to shift
            
            shifted_isi = circshift(isi, randtoshift(q)); % Randomly shifting the ISI
            shifted_isi =  [spikeTimings(1) shifted_isi];
    
            shifted_SpikeTimings = (cumsum(shifted_isi)) / info.dt;
           
            for p = 1:length(shifted_SpikeTimings) % Calculating the shifted sta for each AP
                
                shiftedstavector(:, q) = shiftedstavector(:,q) + Inoise(k, round(shifted_SpikeTimings(p) - 0.5*info.srate) : round(shifted_SpikeTimings(p) + 0.5*info.srate))';
                
            end
            
            
        end
        

        for l = 1:length(SpikeTrain) % Calculating the actual sta for each AP

            stavector(l + counter, :) = Inoise(k, round(SpikeTrain(l) - 0.5*info.srate) : round(SpikeTrain(l) + 0.5*info.srate));

        end
        
        counter = counter + length(SpikeTrain);
        
    else
        continue
        
    end

end

shiftedstavector = shiftedstavector ./ spikecounter;
    
%% Calculating the transfer function

frange = 1:0.3:1000; % Frequency range that will be used

% Averaging across all recordings
css_corr = mean(cvector);
css_lags = mean(lagsvector);

sta = mean(stavector);

shiftedCsr = zeros(size(randtoshift, 1), length(1:0.3:1000));

% frequency dependent Gaussian filtering

counter = 1;
Css = zeros(length(frange), length(css_corr));
Csr = zeros(length(frange), length(css_corr));
for f = frange % Loop over all the frequencies
      
    w = exp( (-(f^2) * (css_lags .^2 )) / 2);
    
    Fexp = exp(-1i * 2 * pi * f .* css_lags);
    
    Css(counter,:) = abs(sum(w .* Fexp .* css_corr));
    
    Csr(counter,:) = abs(sum(w .* Fexp .* sta));
    
    for q = 1:size(shiftedstavector, 2) % Doing the same for the bootstrapped sta
        
        shiftedCsr(q, counter) = abs(sum(w .* Fexp .* shiftedstavector(:, q)'));
        
    end 
    
    counter = counter + 1;
   
end

TF = Csr ./ Css; % The transfer function

cutoff = frange(find(TF < 0.7 * TF(1), 1, 'first')); % Calculating the cut off

shiftedtf =  zeros(size(shiftedCsr,1), size(shiftedCsr,2));

for q = 1:size(shiftedCsr,1)
    
    shiftedtf(q,:) = shiftedCsr(q,:) ./ Css';
    shiftedtf(q,:) = shiftedtf(q,:) ./ TF(1);
    
end

TF = TF ./ TF(1); % Normalizing the transfer function

Csr = abs(Csr) ./ abs(Css(1));
Css = abs(Css) ./ abs(Css(1));

% Calculating significance (5% confidence)

for p = 1:length(frange)
    
    [N, edges] = histcounts(shiftedtf(:,p), 20);
    centroids = edges(1:end-1) + edges(2)/2;
    N = N / sum(N);
    trend = cumsum(N);
    percentile95(p,:) = centroids(find(trend > 0.95, 1, 'first'));
    
end 

%% Plot data

c = [230 25 75; 60 180 75; 255 210 25; 0 130 200; 245 130 48; 145 30 180; 70 240 240; 240 50 230; 128 0 0; 0 0 128]./255;

figure(1)
set(gcf, 'color', 'white');

subplot(421)
plot(css_lags, sta, 'color', c(5,:));
ylim([min(sta)-1 max(sta)])
box off
ax1 = gca;
ax1.YAxis.Visible = 'off';

subplot(423)
plot(css_lags, css_corr, 'color', c(2,:));
ylim([min(css_corr)-10^8 max(css_corr)])
box off
ax1 = gca;
ax1.YAxis.Visible = 'off';

subplot(4,2,[2, 4]);
semilogx(1:0.3:1000, Css, 'color', c(2,:), 'linewidth', 2, 'displayname', 'FFT(iac)');
hold on
semilogx(1:0.3:1000, Csr_norm, 'color', c(5,:), 'linewidth', 2, 'displayname', 'FFT(sta)');
xlabel('Frequency, Hz'); 
ylabel('Normalized Magnitude');
xticklabels({'1', '10', '100', '1000'});
ylim([0 max(tf.Css)])
box off

lgd = legend;
lgd.FontSize = 14;

subplot(4,2,[5:8]);
h(1) = loglog(1:0.3:1000, TF, 'color', c(4,:), 'linewidth', 2);
hold on
h(2) = loglog(frange, percentile95, 'color', c(6,:), 'linewidth', 2);
h(3) = loglog([1 10^3], [0.7 0.7], '--', 'color', 'k', 'linewidth', 1);
ylim([10^-2 max(tf.TF)])

uistack(h(3),'top');
uistack(h(2),'top');
uistack(h(1),'top');

box off
xlabel('Frequency, Hz'); 
ylabel('Transfer Factor');
xticklabels({'1', '10', '100', '1000'});
yticklabels({'0.01', '0.1', '1'})

[caca, hObj] = legend([h(1), h(2), h(3)], 'transfer function' ,'95th percentile', 'cutoff', 'location', 'southeast', 'fontsize', 14);
hL = findobj(hObj, 'type', 'line');
set(hL, 'linewidth', 3);

%% Save data

tf.TF = TF;
tf.STA = sta;
tf.css_corr = css_corr; 
tf.css_lags = css_lags;
tf.resting = V0;
tf.inputResistance = R;
tf.f = f;
tf.FiringFrequency = FiringFrequency;
tf.NumberOfSpikes = spikecounter;
tf.percentile95 = percentile95;
tf.averageSign = mean(shiftedtf(1:50,:));
tf.Css = Css;
tf.Csr = Csr;
tf.Csr_norm = hey;

saveas(figure(1), fullfile(pwd , 'tf'), 'fig');
save('tf.mat', 'tf');