%% cutoff vs onset speed/firing rate/slope at cutoff

clearvars
close all
c = [230 25 75; 60 180 75; 255 210 25; 0 130 200; 245 130 48; 145 30 180; 70 240 240; 240 50 230; 128 0 0; 0 0 128]./255;
% Collecting all the required directories
directories = dir('/Users/stefanobordabossana/Documents/Experiments/analysed cells with tf/*');
directories(1:3) = [];
directories(end) = [];

tf_data = zeros(length(directories), 11);

for i = 1:length(directories) % Loop over different cells
    
    % Loading the relevant data from the transfer function calulcations
    load([directories(i).folder '/' directories(i).name '/tf/tf.mat']);
    tf_data(i,1) = tf.cutoff;
    tf_data(i,2) = tf.NumberOfSpikes;
    tf_data(i,3) = mean(tf.FiringFrequency);
    tf_data(i,4) = tf.slope_fittedtf;
   
    % Loading the relevant data from the AP properties
    load([directories(i).folder '/' directories(i).name '/tf/AP_noise.mat']);
    tf_data(i,5) = AP_noise.slopeatAPonset;
    tf_data(i,6) = AP_noise.speedatAPonset;
    tf_data(i,7) = AP_noise.threshold;
    tf_data(i,8) = AP_noise.amplitude;
    tf_data(i,9) = AP_noise.halfWidth; % FWHM
    tf_data(i,10) = AP_noise.duration;
    
    % Loading the relevant data from tau
    load([directories(i).folder '/' directories(i).name '/tau/01/tau.mat']);
    taucell = struct2cell(tau);
    taumatr = cell2mat(taucell([1 2]));
    
    if max(taumatr) > 100
        tf_data(i,12) = min(taumatr);
    else
        tf_data(i,12) = max(taumatr);
    end
   
end

% Plotting the data
figure(1) % cutoff vs firing rate
set(gcf, 'color', 'white')
hold on

scatter(tf_data(:,3), tf_data(:,1), 100,  c(etypes1,:), 'filled');
ax2 = gca;
ax2.FontSize = 12;
ax2.FontName = 'Arial';
xlabel('Average output firing rate [Hz]');
ylabel('Cutoff frequency [Hz]');

h(1) = scatter(NaN, NaN, 3, c(1,:), 'filled');
h(2) = scatter(NaN, NaN, 3, c(2,:), 'filled');
h(3) = scatter(NaN, NaN, 3, c(3,:), 'filled');
h(4) = scatter(NaN, NaN, 3, c(4,:), 'filled');
h(5) = scatter(NaN, NaN, 3, c(5,:), 'filled');

[~, hObj] = legend([h(1), h(2), h(3), h(4), h(5)], 'cAC' ,'cNAC', 'bNAC', 'cSTUT', 'cIR', 'fontsize', 8, 'linewidth', 1, 'location', 'northeast');
hL = findobj(hObj, 'type', 'line');
set(hL, 'linewidth', 3);

figure(2) % cutoff vs onset speed
set(gcf, 'color', 'white')
hold on

scatter(tf_data(:,6), tf_data(:,1), 100,  c(etypes1,:), 'filled');
xlabel('AP onset speed [ms^{-1}]');
ylabel('Cutoff frequency [Hz]');

h(1) = scatter(NaN, NaN, 1, c(1,:), 'filled');
h(2) = scatter(NaN, NaN, 1, c(2,:), 'filled');
h(3) = scatter(NaN, NaN, 1, c(3,:), 'filled');
h(4) = scatter(NaN, NaN, 1, c(4,:), 'filled');
h(5) = scatter(NaN, NaN, 1, c(5,:), 'filled');

[~, hObj] = legend([h(1), h(2), h(3), h(4), h(5)], 'cAC' ,'cNAC', 'bNAC', 'cSTUT', 'cIR', 'fontsize', 8, 'linewidth', 1, 'location', 'northeast');

figure(3) % cutoff vs alpha
set(gcf, 'color', 'white');
hold on

scatter(tf_data(:,4), tf_data(:,1), 100,  c(etypes1,:), 'filled');
ax2 = gca;
ax2.FontSize = 12;
ax2.FontName = 'Arial';
xlabel('{\alpha}');
ylabel('Cutoff frequency [Hz]');

h(1) = scatter(NaN, NaN, 1, c(1,:), 'filled');
h(2) = scatter(NaN, NaN, 1, c(2,:), 'filled');
h(3) = scatter(NaN, NaN, 1, c(3,:), 'filled');
h(4) = scatter(NaN, NaN, 1, c(4,:), 'filled');
h(5) = scatter(NaN, NaN, 1, c(5,:), 'filled');

[~, hObj] = legend([h(1), h(2), h(3), h(4), h(5)], 'cAC' ,'cNAC', 'bNAC', 'cSTUT', 'cIR', 'fontsize', 14, 'linewidth', 1, 'location', 'southeast');
hL = findobj(hObj, 'type', 'line');
set(hL, 'linewidth', 3);

%% plot distribution histrograms of cutoffs

for i = 1:5

    fig = figure(2);
    box off
   
    h = histfit(data1(etypes1 == i,end), 10, 'kernel');
    x = h(2).XData;
    y = h(2).YData;
    h(2).Color = c(i,:);
    h(1).FaceColor = c(i,:);

    close(fig)
    
    if i <= 3
     
    figure(3)
    set(gcf, 'color', 'white')
    box off
    
    subplot(3,1,i)
    hold on; box off
    h = histogram(data1(etypes1 == i,end), 10);
    h(1).FaceColor = [0.5 0.5 0.5];
    h(1).EdgeColor = [0.5 0.5 0.5];
    plot(x, y, 'color', c(i,:), 'linewidth', 3);
    xlim([0 max(data1(:,end))]);
    ylabel('# cells');
    
    else
        
    figure(4)
    set(gcf, 'color', 'white')
    box off
    
    subplot(3,1,i-3)
    hold on; box off
    h = histogram(data1(etypes1 == i,end), 10);
    h(1).FaceColor = [0.5 0.5 0.5];
    h(1).EdgeColor = [0.5 0.5 0.5];
    plot(x, y, 'color', c(i,:), 'linewidth', 3);
    xlim([0 max(data1(:,end))]);
    ylabel('# cells');
    
    end 
    
    figure(1)
    set(gcf, 'color', 'white')
    box off
    hold on
    
    plot(x, y./max(y), 'color', c(i,:), 'linewidth', 2);
    xlim([0 max(data1(:,end))]); 
    ylabel('Normalized density');
    xlabel('Cutoff frequency [Hz]');
    
end 

hold on
h(1) = loglog([NaN NaN], [NaN NaN], 'color', c(1,:));
h(2) = loglog([NaN NaN], [NaN NaN], 'color', c(2,:));
h(3) = loglog([NaN NaN], [NaN NaN], 'color', c(3,:));
h(4) = loglog([NaN NaN], [NaN NaN], 'color', c(4,:));
h(5) = loglog([NaN NaN], [NaN NaN], 'color', c(5,:));


[~, hObj] = legend([h(1), h(2), h(3), h(4), h(5)], 'cAC' ,'cNAC', 'bNAC', 'cSTUT', 'cIR', 'fontsize', 14, 'linewidth', 1, 'location', 'northeast');
hL = findobj(hObj, 'type', 'line');
set(hL, 'linewidth', 3);

   